@@ -1,10 +1,11 @@
-from gurobipy import *
+from mip import *
 import pandas as pd
 import numpy as np
+
 class MSBM():
     def __init__(self, data: pd.DataFrame, input_cols: list, output_cols: list, un_output_cols: list, vv):
         self.data = data
-        self.N = list(range(0,data.shape[0]))
+        self.K = list(range(0,data.shape[0]))
         self.input = input_cols
         self.output = output_cols
         self.undes = un_output_cols
@@ -40,23 +41,25 @@ class MSBM():
         x_slacks = {p: [] for p in self.input}
         y_slacks= {q: [] for q in self.output}
         u_slacks = {u: [] for u in self.undes}
-        for i in self.N:
-            model = Model('SBMu_model')
-            model.Params.LogToConsole = 0
-            lambdas = model.addVars(self.N,lb=0,vtype=GRB.CONTINUOUS,name='lambdas')
-            slacks_input = model.addVars(self.input,lb=0.0,vtype=GRB.CONTINUOUS, name='slacks_input')
-            slacks_output = model.addVars(self.output,lb=0.0,vtype=GRB.CONTINUOUS, name='slacks_output')
-            slacks_undes = model.addVars(self.undes,lb=0.0,vtype=GRB.CONTINUOUS, name='slacks_undes')
-            t =  model.addVar(lb=0.0,vtype=GRB.CONTINUOUS, name='t')
-            model.addConstrs((quicksum(lambdas[j] * data.loc[j,p] for j in self.N) + slacks_input[p] -  t * data.loc[i,p] == 0 for p in self.input))
-            model.addConstrs((quicksum(lambdas[j] * data.loc[j,q] for j in self.N) - slacks_output[q]-  t * data.loc[i,q] == 0 for q in self.output))
-            model.addConstrs((quicksum(lambdas[j] * data.loc[j,u] for j in self.N) + slacks_undes[u]-  t * data.loc[i,u] == 0 for u in self.undes))
-            model.addConstr(t + 1/(len(self.output)+ len(self.undes))* (quicksum(slacks_output[q] / data.loc[i,q] for q in self.output) + quicksum(slacks_undes[u] / data.loc[i,u] for u in self.undes)) == 1)
+        for i in self.K:
+            model = Model(sense=MINIMIZE, solver_name=CBC)
+            lambdas = [model.add_var(var_type=CONTINUOUS, lb=0) for i in self.K]
+            slacks_input = {i: model.add_var(var_type=CONTINUOUS, lb=0) for i in self.input}
+            slacks_output = {i: model.add_var(var_type=CONTINUOUS, lb=0) for i in self.output}
+            slacks_undes = {i: model.add_var(var_type=CONTINUOUS, lb=0) for i in self.undes}
+            t =  model.add_var(var_type=CONTINUOUS, lb=0)
+            for p in self.input:
+                model += xsum(lambdas[j] * data.loc[j,p] for j in self.K) +slacks_input[p] -  t * data.loc[i,p] == 0
+            for q in self.output:
+                model += xsum(lambdas[j] * data.loc[j,q] for j in self.K) - slacks_output[q]-  t * data.loc[i,q] == 0
+            for u in self.undes:
+                model += xsum(lambdas[j] * data.loc[j,u] for j in self.K) + slacks_undes[u]-  t * data.loc[i,u] == 0
+            model += t + 1/(len(self.output)+ len(self.undes))* (xsum(slacks_output[q] / data.loc[i,q] for q in self.output) + xsum(slacks_undes[u] / data.loc[i,u] for u in self.undes)) == 1
             if self.key == '规模报酬可变VRS':
-                model.addConstr(quicksum(lambdas[j] for j in self.N) == t)
-            model.setObjective(t - 1/len(self.input) * quicksum(slacks_input[p] / data.loc[i,p] for p in self.input))
+                model += xsum(lambdas[j] for j in self.K) == t
+            model.objective = t - 1/len(self.input) * xsum(slacks_input[p] / data.loc[i,p] for p in self.input)
             model.optimize()
-            eff = model.getObjective().getValue()
+            eff = model.objective.x
             x_slack = {p: slacks_input[p].x / t.x if np.abs(slacks_input[p].x / t.x) >= 1e-7 else 0 for p in self.input}
             y_slack = {q: slacks_output[q].x / t.x if np.abs(slacks_output[q].x / t.x) >= 1e-7 else 0 for q in self.output}
             u_slack = {u: slacks_undes[u].x / t.x if np.abs(slacks_undes[u].x / t.x) >= 1e-7 else 0 for u in self.undes}
@@ -73,21 +76,22 @@ class MSBM():
         efficiencies_SBM_model=[]
         x_slacks = {i: [] for i in self.input}
         y_slacks= {j: [] for j in self.output}
-        for i in self.N:
-            model = Model('SBM_model')
-            model.Params.LogToConsole = 0
-            lambdas = model.addVars(self.N,lb=0,vtype=GRB.CONTINUOUS,name='lambdas')
-            slacks_input = model.addVars(self.input,lb=0.0,vtype=GRB.CONTINUOUS, name='slacks_input')
-            slacks_output = model.addVars(self.output,lb=0.0,vtype=GRB.CONTINUOUS, name='slacks_output')
-            t =  model.addVar(lb=0.0,vtype=GRB.CONTINUOUS, name='t')
-            model.addConstrs((quicksum(lambdas[j] * data.loc[j,p] for j in self.N) + slacks_input[p] -  t * data.loc[i,p] == 0 for p in self.input))
-            model.addConstrs((quicksum(lambdas[j] * data.loc[j,q] for j in self.N) - slacks_output[q]-  t * data.loc[i,q] == 0 for q in self.output))
-            model.addConstr(t + (1/len(self.output)) * quicksum(slacks_output[q] / data.loc[i,q] for q in self.output) == 1)
+        for i in self.K:
+            model = Model(sense=MINIMIZE, solver_name=CBC)
+            lambdas = [model.add_var(var_type=CONTINUOUS, lb=0) for i in self.K]
+            slacks_input = {i: model.add_var(var_type=CONTINUOUS, lb=0) for i in self.input}
+            slacks_output = {i: model.add_var(var_type=CONTINUOUS, lb=0) for i in self.output}
+            t =  model.add_var(var_type=CONTINUOUS, lb=0)
+            for p in self.input:
+                model += xsum(lambdas[j] * data.loc[j,p] for j in self.K) + slacks_input[p] -  t * data.loc[i,p] == 0
+            for q in self.output:
+                model += xsum(lambdas[j] * data.loc[j,q] for j in self.K) - slacks_output[q]-  t * data.loc[i,q] == 0
+            model += t + (1/len(self.output)) * xsum(slacks_output[q] / data.loc[i,q] for q in self.output) == 1
             if self.key == '规模报酬可变VRS':
-                model.addConstr(quicksum(lambdas[j] for j in self.N) == t)
-            model.setObjective(t - 1/len(self.input) * quicksum(slacks_input[p] / data.loc[i,p] for p in self.input))
+                model += xsum(lambdas[j] for j in self.K) == t
+            model.objective = t - 1/len(self.input) * xsum(slacks_input[p] / data.loc[i,p] for p in self.input)
             model.optimize()
-            eff = model.getObjective().getValue()
+            eff = model.objective.x
             x_slack = {p: slacks_input[p].x / t.x if np.abs(slacks_input[p].x / t.x) >= 1e-7 else 0 for p in self.input}
             y_slack = {q: slacks_output[q].x / t.x if np.abs(slacks_output[q].x / t.x) >= 1e-7 else 0 for q in self.output}
             for s_input in self.input:
@@ -95,4 +99,12 @@ class MSBM():
             for s_output in self.output:
                 y_slacks[s_output].append(y_slack[s_output])       
             efficiencies_SBM_model.append(eff) 
-        return efficiencies_SBM_model, x_slacks, y_slacks
\ No newline at end of file
+        return efficiencies_SBM_model, x_slacks, y_slacks
+
+if __name__ == '__main__':
+    data = pd.read_excel(r'deatool\MMODEL\测试数据.xlsx')
+    input_cols = ['输入1','输入2']
+    output_cols = ['输出1','输出2']
+    un_output_cols = ['非期望输出1']
+    model = MSBM(data=data, input_cols = input_cols, output_cols = output_cols,un_output_cols = un_output_cols, vv = '规模报酬不变CRS')
+    print(model.res())
\ No newline at end of file
